! gravity2.f90
! Compute gravitational potential (multipole).
! Miroslav Broz (miroslav.broz@email.cz), Aug 7th 2020

program gravity2

! modules
use read_elem_module
use read_face_module
use read_node_module
use volume_module
use multipole_module
use potential_module

implicit none

! constants
integer, parameter :: npole = 10
double precision, parameter :: pi = 4.d0*atan(1.d0)

! variables
character(len=255) :: f_elem, f_face, f_node
double precision :: rho, unit, scale
double precision :: x1, x2, dx, y1, y2, dy, z1, z2, dz

integer :: l, m, j
integer :: nrot
double precision :: V, capr, capm
double precision :: x, y, z, U
double precision, dimension(3) :: r
real :: t1, t2
double precision, dimension(0:npole,0:npole) :: Clm, Slm
integer, dimension(:,:), pointer :: elems, faces
double precision, dimension(:,:), pointer :: nodes
double precision, dimension(:), pointer :: vols

! read input
write(*,*) '# elem file:'
read(*,*) f_elem
write(*,*) '# face file:'
read(*,*) f_face
write(*,*) '# node file:'
read(*,*) f_node
write(*,*) '# density [kg m^-3]:'
read(*,*) rho
write(*,*) '# coordinate range and step [x1, x2, dx, y1, y2, dy, z1, z2, dz]:'
read(*,*) x1, x2, dx, y1, y2, dy, z1, z2, dz
write(*,*) '# unit length [m]:'
read(*,*) unit
write(*,*) '# scale []:'
read(*,*) scale
write(*,*)

! write input
write(*,*) '# f_elem = ', trim(f_elem)
write(*,*) '# f_face = ', trim(f_face)
write(*,*) '# f_node = ', trim(f_node)
write(*,*) '# rho = ', rho, ' kg m^-3'
write(*,*) '# x1 = ', x1, ' u.'
write(*,*) '# x2 = ', x2, ' u.'
write(*,*) '# dx = ', dx, ' u.'
write(*,*) '# y1 = ', y1, ' u.'
write(*,*) '# y2 = ', y2, ' u.'
write(*,*) '# dy = ', dy, ' u.'
write(*,*) '# z1 = ', z1, ' u.'
write(*,*) '# z2 = ', z2, ' u.'
write(*,*) '# dz = ', dz, ' u.'
write(*,*) '# unit = ', unit, ' m'
write(*,*) '# scale = ', scale

! input files
call read_elem(f_elem, elems)
call read_face(f_face, faces)
call read_node(f_node, nodes)

write(*,*) '# nelems = ', size(elems,1)
write(*,*) '# nfaces = ', size(faces,1)
write(*,*) '# nnodes = ', size(nodes,1)

allocate(vols(size(elems,1)))

! unit conversion
nodes = nodes*unit*scale  ! km -> m
r = r*unit
x1 = x1*unit
y1 = y1*unit
z1 = z1*unit
x2 = x2*unit
y2 = y2*unit
z2 = z2*unit
dx = dx*unit
dy = dy*unit
dz = dz*unit

! volume
V = volume(elems, nodes, vols)
capr = (V/(4.d0/3.d0*pi))**(1.d0/3.d0)
capm = V * rho

write(*,*) '# V = ', V, ' m^3'
write(*,*) '# R = ', capr, ' m = ', capr/unit, ' [unit]'
write(*,*) '# 2R = ', 2.d0*capr, ' m = ', 2.d0*capr/unit, ' [unit]'
write(*,*) '# M = ', capm, ' kg'
write(*,*) '# M = ', 4.64d18, ' kg (Deschamps etal. 2011)'
write(*,*)

! high-order multipoles
call multipole(elems, nodes, capm, capr, rho, Clm, Slm)

do l = 0, size(Clm,1)-1
  do m = 0, l
    write(*,*) '# Clm(', l, ',', m, ') = ', Clm(l,m)
    if (m.gt.0) then
      write(*,*) '# Slm(', l, ',', m, ') = ', Slm(l,m)
    endif
  enddo
  write(*,*)
enddo

! potential
write(*,*) '# x y z U'
call cpu_time(t1)
z = z1
do while (z.le.z2)
  y = y1
  do while (y.le.y2)
    x = x1
    do while (x.le.x2)
      r = (/x, y, z/)
      U = potential(Clm, Slm, npole, capm, capr, r)
      write(*,*) x/unit, y/unit, z/unit, U
      x = x+dx
    enddo
    write(*,*)
    y = y+dy
  enddo
  z = z+dz
enddo
call cpu_time(t2)
write(*,*) '# cpu_time = ', t2-t1, ' s'
write(*,*)

stop
end program gravity2


