! potential.f90
! Compute gravitational potential using a multipole expansion.
! Miroslav Broz (miroslav.broz@email.cz), Nov 18th 2019

module potential_module

contains

double precision function potential(Clm, Slm, npole, capm, capr, r)

use const_module
use legendre_module

implicit none

double precision, dimension(0:,0:) :: Clm, Slm
integer :: npole
double precision :: capm, capr
double precision, dimension(3) :: r

integer l, m
double precision :: absr, costheta, phi, U

U = 0.d0

absr = sqrt(dot_product(r,r))
costheta = r(3)/absr
phi = atan2(r(2),r(1))

do l = 0, npole
  do m = 0, l
    U = U + (capr/absr)**l * Plm(l,m,costheta) * (Clm(l,m)*cos(m*phi) + Slm(l,m)*sin(m*phi))  ! Sidlichovsky; k = GM != m
  enddo
enddo

U = -U*G*capm/absr
potential = U

return

end function potential


function a_g_mp(Clm, Slm, npole, capm, capr, r)

use const_module
use legendre_module

implicit none

double precision, dimension(3) :: a_g_mp
double precision, dimension(0:,0:) :: Clm, Slm
integer :: npole
double precision :: capm, capr
double precision, dimension(3) :: r

integer l, m
double precision :: absr, theta, phi, costheta, sintheta
double precision :: dU_dr, dU_dtheta, dU_dphi, Plm_
double precision, dimension(3) :: a_g

a_g = 0.d0

absr = sqrt(dot_product(r,r))
costheta = r(3)/absr
sintheta = sqrt(1.d0-costheta**2)
theta = acos(costheta)
phi = atan2(r(2),r(1))

! U = G*capm/absr * \sum_l=0^\infty \sum_m=0^l (capr/absr)**l * Plm(l,m,costheta) * (Clm(l,m)*cos(m*phi) + Slm(l,m)*sin(m*phi))  ! Sidlichovsky; k = GM != m

do l = 0, npole
  do m = 0, l

    Plm_ = Plm(l,m,costheta)
    dU_dr = capr**l * (-l-1)*absr**(-l-2) * Plm_ * (Clm(l,m)*cos(m*phi) + Slm(l,m)*sin(m*phi))
    dU_dtheta = capr**l * absr**(-l-1) * dPlm_dx(l,m,costheta,Plm_)*sintheta * (Clm(l,m)*cos(m*phi) + Slm(l,m)*sin(m*phi))  ! minus or NOT?
    dU_dphi = capr**l * absr**(-l-1) * Plm_ * (-Clm(l,m)*sin(m*phi)*m + Slm(l,m)*cos(m*phi)*m)

    a_g(1) = a_g(1) + dU_dr
    a_g(2) = a_g(2) + dU_dtheta/absr
    if (sintheta.ne.0.d0) then
      a_g(3) = a_g(3) + dU_dphi/(absr*sintheta)
    endif
  enddo
enddo

a_g = a_g*G*capm
a_g = spherical_cartesian([absr, theta, phi], a_g)
a_g_mp = a_g

return

end function a_g_mp


function spherical_cartesian(r, a)

double precision, dimension(3) :: spherical_cartesian, r, a
double precision, dimension(3) :: tmp
double precision :: sintheta, costheta, sinphi, cosphi

!a(2) = 1.d0
!a(3) = 0.5d0
!write(*,*) 'a_g = ', a, ' m s^-2 (spherical)'
!write(*,*) 'r = ', r, ' m (spherical)'

sintheta = sin(r(2))
costheta = cos(r(2))
sinphi = sin(r(3))
cosphi = cos(r(3))

tmp(1) = a(1)*sintheta*cosphi - a(2)*costheta*cosphi - a(3)*sinphi
tmp(2) = a(1)*sintheta*sinphi - a(2)*costheta*sinphi + a(3)*cosphi
tmp(3) = a(1)*costheta        + a(2)*sintheta

spherical_cartesian = tmp
return

end function spherical_cartesian

end module potential_module


